import { Module } from '@nestjs/common';
import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { UserController } from './logger/adapters/primaries/user.controller';
import { WinstonModule } from 'nest-winston';
import * as winston from 'winston';
import { AuthController } from './logger/adapters/primaries/auth.controller';
import { ConfigurationController } from './logger/adapters/primaries/configuration.controller';
import { TerritoryController } from './logger/adapters/primaries/territory.controller';

@Module({
  imports: [
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        exchanges: [
          {
            name: 'logging',
            type: 'topic',
          },
        ],
        uri: configService.get<string>('RMQ_URI'),
        enableControllerDiscovery: true,
      }),
      inject: [ConfigService],
    }),
    WinstonModule.forRoot({
      levels: winston.config.syslog.levels,
      transports: [],
    }),
  ],
  controllers: [
    AuthController,
    UserController,
    ConfigurationController,
    TerritoryController,
  ],
  providers: [],
  exports: [],
})
export class LoggerModule {}
