import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Controller, Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import { level } from './logger/level.enum';
import loggerOptions from './logger/logger';

@Controller()
export class AuthController {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'auth.create.crit',
    queue: 'logging-auth-create-crit',
  })
  public async authCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.crit, 'critical', 'create'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'auth.delete.crit',
    queue: 'logging-auth-delete-crit',
  })
  public async authDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.crit, 'critical', 'delete'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'auth.username.add.warning',
    queue: 'logging-auth-username-add-warning',
  })
  public async authUsernameAddedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-add'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'auth.username.delete.warning',
    queue: 'logging-auth-username-delete-warning',
  })
  public async authUsernameDeletedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-delete'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'auth.password.update.warning',
    queue: 'logging-auth-password-update-warning',
  })
  public async authPasswordUpdatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'password-update'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'auth.username.update.warning',
    queue: 'logging-auth-username-update-warning',
  })
  public async authUsernameUpdatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('auth', level.warning, 'warning', 'username-update'),
    );
    this.logger.warning(JSON.parse(message));
  }
}
