import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Controller, Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import { level } from './logger/level.enum';
import loggerOptions from './logger/logger';

@Controller()
export class ConfigurationController {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.create.info',
    queue: 'logging-configuration-create-info',
  })
  public async configurationCreatedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.info, 'info', 'create'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.create.warning',
    queue: 'logging-configuration-create-warning',
  })
  public async configurationCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.warning, 'warning', 'create'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.create.crit',
    queue: 'logging-configuration-create-crit',
  })
  public async configurationCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.crit, 'critical', 'create'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.delete.info',
    queue: 'logging-configuration-delete-info',
  })
  public async configurationDeletedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.info, 'info', 'delete'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.delete.crit',
    queue: 'logging-configuration-delete-crit',
  })
  public async configurationDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.crit, 'critical', 'delete'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.read.warning',
    queue: 'logging-configuration-read-warning',
  })
  public async configurationReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.warning, 'warning', 'read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.update.info',
    queue: 'logging-configuration-update-info',
  })
  public async configurationUpdatedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.info, 'info', 'update'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'configuration.update.crit',
    queue: 'logging-configuration-update-crit',
  })
  public async configurationUpdatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.crit, 'critical', 'update'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
