import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Controller, Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import { level } from './logger/level.enum';
import loggerOptions from './logger/logger';

@Controller()
export class UserController {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.create.info',
    queue: 'logging-user-create-info',
  })
  public async userCreatedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('user', level.info, 'info', 'create'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.create.warning',
    queue: 'logging-user-create-warning',
  })
  public async userCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.warning, 'warning', 'create'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.create.crit',
    queue: 'logging-user-create-crit',
  })
  public async userCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'create'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.delete.info',
    queue: 'logging-user-delete-info',
  })
  public async userDeletedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('user', level.info, 'info', 'delete'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.delete.crit',
    queue: 'logging-user-delete-crit',
  })
  public async userDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'delete'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.read.warning',
    queue: 'logging-user-read-warning',
  })
  public async userReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.warning, 'warning', 'read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.update.info',
    queue: 'logging-user-update-info',
  })
  public async userUpdatedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('user', level.info, 'info', 'update'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    exchange: 'logging',
    routingKey: 'user.update.crit',
    queue: 'logging-user-update-crit',
  })
  public async userUpdatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'update'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
